Exception: Aide::Login_Error
- Inherits:
-
StandardError
- Object
- StandardError
- Aide::Login_Error
- Defined in:
- lib/aide/errors.rb
Overview
Login_Error represents a problem with the credentials given to an instance of Aide::Bot.
Login_Error represents a problem with the credentials given to an instance of Aide::Bot.